@@ -1,19 +1,23 @@
 use crate::{
     components::{
-        CommandInfo, CommitComponent, Component, DiffComponent,
-        HelpComponent, IndexComponent,
+        ChangesComponent, CommandBlocking, CommandInfo,
+        CommitComponent, Component, DiffComponent, DrawableComponent,
+        EventUpdate, HelpComponent, ResetComponent,
     },
-    keys, strings,
+    keys,
+    queue::{InternalEvent, Queue},
+    strings,
 };
 use asyncgit::{
     current_tick, sync, AsyncDiff, AsyncNotification, AsyncStatus,
-    DiffParams,
+    DiffParams, CWD,
 };
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
 use itertools::Itertools;
 use log::trace;
 use std::{borrow::Cow, path::Path};
+use strings::commands;
 use tui::{
     backend::Backend,
     layout::{Alignment, Constraint, Direction, Layout, Rect},
@@ -32,7 +36,7 @@ enum DiffTarget {
 ///
 #[derive(PartialEq)]
 enum Focus {
-    Status,
+    WorkDir,
     Diff,
     Stage,
 }
@@ -42,33 +46,47 @@ pub struct App {
     focus: Focus,
     diff_target: DiffTarget,
     do_quit: bool,
+    reset: ResetComponent,
     commit: CommitComponent,
     help: HelpComponent,
-    index: IndexComponent,
-    index_wd: IndexComponent,
+    index: ChangesComponent,
+    index_wd: ChangesComponent,
     diff: DiffComponent,
     git_diff: AsyncDiff,
     git_status: AsyncStatus,
+    current_commands: Vec<CommandInfo>,
+    queue: Queue,
 }
 
 // public interface
 impl App {
     ///
     pub fn new(sender: Sender<AsyncNotification>) -> Self {
+        let queue = Queue::default();
         Self {
-            focus: Focus::Status,
+            focus: Focus::WorkDir,
             diff_target: DiffTarget::WorkingDir,
             do_quit: false,
+            reset: ResetComponent::new(queue.clone()),
             commit: CommitComponent::default(),
             help: HelpComponent::default(),
-            index_wd: IndexComponent::new(
+            index_wd: ChangesComponent::new(
                 strings::TITLE_STATUS,
                 true,
+                true,
+                queue.clone(),
+            ),
+            index: ChangesComponent::new(
+                strings::TITLE_INDEX,
+                false,
+                false,
+                queue.clone(),
             ),
-            index: IndexComponent::new(strings::TITLE_INDEX, false),
             diff: DiffComponent::default(),
             git_diff: AsyncDiff::new(sender.clone()),
             git_status: AsyncStatus::new(sender),
+            current_commands: Vec::new(),
+            queue,
         }
     }
 
@@ -130,76 +148,46 @@ impl App {
         Self::draw_commands(
             f,
             chunks_main[2],
-            self.commands().as_slice(),
+            self.current_commands.as_slice(),
         );
 
         self.commit.draw(f, f.size());
         self.help.draw(f, f.size());
+        self.reset.draw(f, f.size());
     }
 
     ///
     pub fn event(&mut self, ev: Event) {
         trace!("event: {:?}", ev);
 
-        if self.commit.is_visible() && self.commit.event(ev) {
-            if !self.commit.is_visible() {
-                self.update();
+        if let Some(e) =
+            Self::event_pump(ev, self.components_mut().as_mut_slice())
+        {
+            match e {
+                EventUpdate::All => self.update(),
+                EventUpdate::Commands => self.update_commands(),
+                EventUpdate::Diff => self.update_diff(),
+                _ => (),
             }
-            return;
-        } else if self.help.is_visible() {
-            self.help.event(ev);
-            return;
+        } else if let Event::Key(k) = ev {
+            match k {
+                keys::EXIT_1 | keys::EXIT_2 => self.do_quit = true,
+                keys::FOCUS_WORKDIR => {
+                    self.switch_focus(Focus::WorkDir)
+                }
+                keys::FOCUS_STAGE => self.switch_focus(Focus::Stage),
+                keys::FOCUS_RIGHT => self.switch_focus(Focus::Diff),
+                keys::FOCUS_LEFT => {
+                    self.switch_focus(match self.diff_target {
+                        DiffTarget::Stage => Focus::Stage,
+                        DiffTarget::WorkingDir => Focus::WorkDir,
+                    })
+                }
+                _ => (),
+            };
         }
 
-        if !self.commit.is_visible() {
-            if self.index.event(ev) {
-                self.update_diff();
-                return;
-            }
-            if self.index_wd.event(ev) {
-                self.update_diff();
-                return;
-            }
-            if self.diff.event(ev) {
-                return;
-            }
-
-            if let Event::Key(k) = ev {
-                match k {
-                    keys::EXIT_1 | keys::EXIT_2 => {
-                        self.do_quit = true
-                    }
-                    keys::FOCUS_STATUS => {
-                        self.switch_focus(Focus::Status)
-                    }
-                    keys::OPEN_HELP => self.help.show(),
-                    keys::FOCUS_STAGE => {
-                        self.switch_focus(Focus::Stage)
-                    }
-                    keys::FOCUS_RIGHT => {
-                        self.switch_focus(Focus::Diff)
-                    }
-                    keys::FOCUS_LEFT => {
-                        self.switch_focus(match self.diff_target {
-                            DiffTarget::Stage => Focus::Stage,
-                            DiffTarget::WorkingDir => Focus::Status,
-                        })
-                    }
-                    keys::STATUS_STAGE_FILE => {
-                        self.index_add_remove();
-                        self.update();
-                    }
-                    keys::STATUS_RESET_FILE => {
-                        self.index_reset();
-                        self.update();
-                    }
-                    keys::OPEN_COMMIT if !self.index.is_empty() => {
-                        self.commit.show();
-                    }
-                    _ => (),
-                };
-            }
-        }
+        self.process_queue();
     }
 
     ///
@@ -257,104 +245,151 @@ impl App {
         }
     }
 
+    fn update_commands(&mut self) {
+        self.help.set_cmds(self.commands(true));
+        self.current_commands = self.commands(false);
+        self.current_commands.sort_by_key(|e| e.order);
+    }
+
     fn update_status(&mut self) {
         let status = self.git_status.last();
         self.index.update(&status.stage);
         self.index_wd.update(&status.work_dir);
 
         self.update_diff();
-
-        self.help.set_cmds(self.commands());
+        self.commit.set_stage_empty(self.index.is_empty());
+        self.update_commands();
     }
 
-    fn commands(&self) -> Vec<CommandInfo> {
-        let mut res = self.commit.commands();
-        res.extend(self.help.commands());
+    fn process_queue(&mut self) {
+        loop {
+            let front = self.queue.borrow_mut().pop_front();
+            if let Some(e) = front {
+                self.process_internal_event(&e);
+            } else {
+                break;
+            }
+        }
+        self.queue.borrow_mut().clear();
+    }
 
-        res.extend(self.index.commands());
-        res.extend(self.index_wd.commands());
-        res.extend(self.diff.commands());
+    fn process_internal_event(&mut self, ev: &InternalEvent) {
+        match ev {
+            InternalEvent::ResetFile(p) => {
+                if sync::reset_workdir(CWD, Path::new(p.as_str())) {
+                    self.update();
+                }
+            }
+            InternalEvent::ConfirmResetFile(p) => {
+                self.reset.open_for_path(p);
+                self.update_commands();
+            }
+        };
+    }
+
+    fn commands(&self, force_all: bool) -> Vec<CommandInfo> {
+        let mut res = Vec::new();
+
+        for c in self.components() {
+            if c.commands(&mut res, force_all)
+                != CommandBlocking::PassingOn
+                && !force_all
+            {
+                break;
+            }
+        }
+
+        let main_cmds_available = !self.commit.is_visible()
+            && !self.help.is_visible()
+            && !self.reset.is_visible();
 
         {
-            let main_cmds_available =
-                !self.commit.is_visible() && !self.help.is_visible();
-
-            res.push(CommandInfo::new(
-                strings::COMMIT_CMD_OPEN,
-                !self.index.is_empty(),
-                main_cmds_available,
-            ));
-
-            {
-                let main_cmds_index_wd_focused_availale =
-                    main_cmds_available && self.index_wd.focused();
-
-                let some_selection =
-                    self.index_wd.selection().is_some();
-                res.push(CommandInfo::new(
-                    strings::CMD_STATUS_STAGE,
-                    some_selection,
-                    main_cmds_index_wd_focused_availale,
-                ));
-                res.push(CommandInfo::new(
-                    strings::CMD_STATUS_RESET,
-                    some_selection,
-                    main_cmds_index_wd_focused_availale,
-                ));
-
-                res.push(CommandInfo::new(
-                    strings::CMD_STATUS_UNSTAGE,
-                    self.index.selection().is_some(),
-                    main_cmds_available && self.index.focused(),
-                ));
-            }
-
             {
                 let focus_on_stage = self.focus == Focus::Stage;
                 let focus_not_diff = self.focus != Focus::Diff;
-                res.push(CommandInfo::new_hidden(
-                    strings::CMD_STATUS_FOCUS_UNSTAGED,
-                    true,
-                    main_cmds_available
-                        && focus_on_stage
-                        && !focus_not_diff,
-                ));
-                res.push(CommandInfo::new_hidden(
-                    strings::CMD_STATUS_FOCUS_STAGED,
-                    true,
-                    main_cmds_available
-                        && !focus_on_stage
-                        && !focus_not_diff,
-                ));
+                res.push(
+                    CommandInfo::new(
+                        commands::STATUS_FOCUS_UNSTAGED,
+                        true,
+                        main_cmds_available
+                            && focus_on_stage
+                            && !focus_not_diff,
+                    )
+                    .hidden(),
+                );
+                res.push(
+                    CommandInfo::new(
+                        commands::STATUS_FOCUS_STAGED,
+                        true,
+                        main_cmds_available
+                            && !focus_on_stage
+                            && !focus_not_diff,
+                    )
+                    .hidden(),
+                );
             }
             {
                 let focus_on_diff = self.focus == Focus::Diff;
                 res.push(CommandInfo::new(
-                    strings::CMD_STATUS_LEFT,
+                    commands::STATUS_FOCUS_LEFT,
                     true,
                     main_cmds_available && focus_on_diff,
                 ));
                 res.push(CommandInfo::new(
-                    strings::CMD_STATUS_RIGHT,
+                    commands::STATUS_FOCUS_RIGHT,
                     true,
                     main_cmds_available && !focus_on_diff,
                 ));
             }
-            res.push(CommandInfo::new(
-                strings::CMD_STATUS_HELP,
-                true,
-                main_cmds_available,
-            ));
-            res.push(CommandInfo::new(
-                strings::CMD_STATUS_QUIT,
-                true,
-                main_cmds_available,
-            ));
+
+            res.push(
+                CommandInfo::new(
+                    commands::QUIT,
+                    true,
+                    main_cmds_available,
+                )
+                .order(100),
+            );
         }
 
         res
     }
 
+    fn components(&self) -> Vec<&dyn Component> {
+        vec![
+            &self.reset,
+            &self.commit,
+            &self.help,
+            &self.index,
+            &self.index_wd,
+            &self.diff,
+        ]
+    }
+
+    fn components_mut(&mut self) -> Vec<&mut dyn Component> {
+        vec![
+            &mut self.reset,
+            &mut self.commit,
+            &mut self.help,
+            &mut self.index,
+            &mut self.index_wd,
+            &mut self.diff,
+        ]
+    }
+
+    fn event_pump(
+        ev: Event,
+        components: &mut [&mut dyn Component],
+    ) -> Option<EventUpdate> {
+        for c in components {
+            if let Some(u) = c.event(ev) {
+                return Some(u);
+            }
+        }
+
+        None
+    }
+
     fn draw_commands<B: Backend>(
         f: &mut Frame<B>,
         r: Rect,
@@ -375,7 +410,7 @@ impl App {
             .filter_map(|c| {
                 if c.show_in_quickbar() {
                     Some(Text::Styled(
-                        Cow::from(c.name.clone()),
+                        Cow::from(c.text.name),
                         if c.enabled {
                             style_enabled
                         } else {
@@ -398,7 +433,7 @@ impl App {
             self.focus = f;
 
             match self.focus {
-                Focus::Status => {
+                Focus::WorkDir => {
                     self.set_diff_target(DiffTarget::WorkingDir);
                     self.diff.focus(false);
                 }
@@ -425,34 +460,4 @@ impl App {
         self.index_wd.focus_select(!is_stage);
         self.index.focus_select(is_stage);
     }
-
-    fn index_add_remove(&mut self) {
-        if self.diff_target == DiffTarget::WorkingDir {
-            if let Some(i) = self.index_wd.selection() {
-                let path = Path::new(i.path.as_str());
-
-                if sync::stage_add(path) {
-                    self.update();
-                }
-            }
-        } else if let Some(i) = self.index.selection() {
-            let path = Path::new(i.path.as_str());
-
-            if sync::stage_reset(path) {
-                self.update();
-            }
-        }
-    }
-
-    fn index_reset(&mut self) {
-        if self.index_wd.focused() {
-            if let Some(i) = self.index_wd.selection() {
-                let path = Path::new(i.path.as_str());
-
-                if sync::index_reset(path) {
-                    self.update();
-                }
-            }
-        }
-    }
 }
