@@ -1,10 +1,12 @@
+use super::{CommandBlocking, DrawableComponent, EventUpdate};
 use crate::{
     components::{CommandInfo, Component},
     strings,
 };
 use asyncgit::{hash, Diff, DiffLine, DiffLineType};
 use crossterm::event::{Event, KeyCode};
-use std::borrow::Cow;
+use std::{borrow::Cow, cmp, convert::TryFrom};
+use strings::commands;
 use tui::{
     backend::Backend,
     layout::{Alignment, Rect},
@@ -27,7 +29,7 @@ pub struct DiffComponent {
 impl DiffComponent {
     ///
     fn can_scroll(&self) -> bool {
-        self.diff.0.len() > 1
+        self.diff.1 > 1
     }
     ///
     pub fn current(&self) -> (String, bool) {
@@ -37,6 +39,7 @@ impl DiffComponent {
     pub fn clear(&mut self) {
         self.current.0.clear();
         self.diff = Diff::default();
+        self.current_hash = 0;
     }
     ///
     pub fn update(
@@ -57,8 +60,10 @@ impl DiffComponent {
 
     fn scroll(&mut self, inc: bool) {
         if inc {
-            self.scroll =
-                self.scroll.checked_add(1).unwrap_or(self.scroll);
+            self.scroll = cmp::min(
+                self.diff.1.saturating_sub(1),
+                self.scroll.saturating_add(1),
+            );
         } else {
             self.scroll = self.scroll.saturating_sub(1);
         }
@@ -79,7 +84,7 @@ impl DiffComponent {
                 break;
             }
 
-            let hunk_len = hunk.0.len() as u16;
+            let hunk_len = u16::try_from(hunk.0.len()).unwrap();
             let hunk_min = line_cursor;
             let hunk_max = line_cursor + hunk_len;
 
@@ -131,6 +136,7 @@ impl DiffComponent {
                     Color::Reset
                 })
                 .fg(Color::DarkGray);
+
             if end_of_hunk {
                 text.push(Text::Styled(
                     Cow::from(symbols::line::BOTTOM_LEFT),
@@ -166,13 +172,18 @@ impl DiffComponent {
             .modifier(Modifier::BOLD);
 
         let filled = if selected {
+            // selected line
             format!(
                 "{:w$}\n",
-                line.content.trim_end(),
+                line.content.trim_matches('\n'),
                 w = width as usize
             )
-        } else {
+        } else if line.content.matches('\n').count() == 1 {
+            // regular line, no selection (cheapest)
             line.content.clone()
+        } else {
+            // weird eof missing eol line
+            format!("{}\n", line.content.trim_matches('\n'))
         };
         let content = Cow::from(filled);
 
@@ -210,12 +221,12 @@ impl DiffComponent {
     }
 }
 
-impl Component for DiffComponent {
+impl DrawableComponent for DiffComponent {
     fn draw<B: Backend>(&self, f: &mut Frame<B>, r: Rect) {
-        let mut style_border = Style::default();
+        let mut style_border = Style::default().fg(Color::DarkGray);
         let mut style_title = Style::default();
         if self.focused {
-            style_border = style_border.fg(Color::Green);
+            style_border = style_border.fg(Color::Gray);
             style_title = style_title.modifier(Modifier::BOLD);
         }
 
@@ -230,33 +241,41 @@ impl Component for DiffComponent {
             .alignment(Alignment::Left)
             .render(f, r);
     }
+}
 
-    fn commands(&self) -> Vec<CommandInfo> {
-        vec![CommandInfo::new(
-            strings::CMD_SCROLL,
+impl Component for DiffComponent {
+    fn commands(
+        &self,
+        out: &mut Vec<CommandInfo>,
+        _force_all: bool,
+    ) -> CommandBlocking {
+        out.push(CommandInfo::new(
+            commands::SCROLL,
             self.can_scroll(),
             self.focused,
-        )]
+        ));
+
+        CommandBlocking::PassingOn
     }
 
-    fn event(&mut self, ev: Event) -> bool {
+    fn event(&mut self, ev: Event) -> Option<EventUpdate> {
         if self.focused {
             if let Event::Key(e) = ev {
                 return match e.code {
                     KeyCode::Down => {
                         self.scroll(true);
-                        true
+                        Some(EventUpdate::None)
                     }
                     KeyCode::Up => {
                         self.scroll(false);
-                        true
+                        Some(EventUpdate::None)
                     }
-                    _ => false,
+                    _ => None,
                 };
             }
         }
 
-        false
+        None
     }
 
     fn focused(&self) -> bool {
